//
//  AFURLConnectionOperation+Performance.m
//  YH_AFNetworking
//
//  Created by 王钱钧 on 15/12/23.
//  Copyright © 2015年 YOHO. All rights reserved.
//

#import "AFURLConnectionOperation+Performance.h"
#import "Aspects.h"
#import "YH_Analytics.h"


@implementation AFURLConnectionOperation (Performance)

+ (void)addOperationMonitor
{
    //  start
    [AFURLConnectionOperation aspect_hookSelector:@selector(operationDidStart) withOptions:AspectPositionBefore usingBlock:^(id<AspectInfo>aspectInfo){
        AFURLConnectionOperation *op = [aspectInfo instance];
        op.startTS = [[NSDate date]timeIntervalSince1970];
    } error:NULL];
    
    // finish
    [AFURLConnectionOperation aspect_hookSelector:@selector(connectionDidFinishLoading:) withOptions:AspectPositionAfter usingBlock:^(id<AspectInfo>aspectInfo){
        AFURLConnectionOperation *op = [aspectInfo instance];
        
        op.finishTS = [[NSDate date]timeIntervalSince1970];
        
        NSInteger status = -1;
        long deltaTS = (op.finishTS - op.startTS) * 1000;
        NSString *url = [AFURLConnectionOperation urlWithOperation:op];
        NSString *opAddr = [NSString stringWithFormat:@"%p", op];
        
        if (op.response) {
            if ([op.response isKindOfClass:[NSHTTPURLResponse class]]) {
                NSHTTPURLResponse *response = (NSHTTPURLResponse *)op.response;
                status = [response statusCode];
            }
            if (status == 200) {
                // success
                status = 1;
            }
            [[YH_Analytics sharedInstance]logPerformanceWithType:kPerformanceTypeAPICall
                                                      parameters:@{JsonKeyPerformanceEvtEID:opAddr?:@"",
                                                                   JsonKeyPerformanceURL: url?:@"",
                                                                   JsonKeyPerformanceTS: @((long)(op.startTS*1000)),
                                                                   JsonKeyPerformanceCTS:@(deltaTS),
                                                                   JsonKeyPerformanceSTATUS : @(status)
                                                                   }];
        }
        
    } error:NULL];
    
    
    // error
    [AFURLConnectionOperation aspect_hookSelector:@selector(connection:didFailWithError:) withOptions:AspectPositionAfter usingBlock:^(id<AspectInfo>aspectInfo){
        AFURLConnectionOperation *op = [aspectInfo instance];
        
        op.errorTS = [[NSDate date]timeIntervalSince1970];
        long deltaTS = (op.errorTS - op.startTS) * 1000;
        NSString *url = [AFURLConnectionOperation urlWithOperation:op];
        
        // error code default is `-1`
        NSInteger errorCode = -1;
        if (op.error) {
            errorCode = [op.error code];
        }
        NSString *opAddr = [NSString stringWithFormat:@"%p", op];
        [[YH_Analytics sharedInstance]logPerformanceWithType:kPerformanceTypeAPICall
                                                  parameters:@{JsonKeyPerformanceEvtEID:opAddr?:@"",
                                                               JsonKeyPerformanceURL: url?:@"",
                                                               JsonKeyPerformanceTS: @((long)(op.startTS*1000)),
                                                               JsonKeyPerformanceCTS:@(deltaTS),
                                                               JsonKeyPerformanceSTATUS : @(errorCode)
                                                               }];
    } error:NULL];
    
    
    // cancel
    [AFURLConnectionOperation aspect_hookSelector:@selector(cancelConnection) withOptions:AspectPositionAfter usingBlock:^(id<AspectInfo>aspectInfo){
        AFURLConnectionOperation *op = [aspectInfo instance];
        
        op.cancelTS = [[NSDate date]timeIntervalSince1970];
        
        long deltaTS = (op.cancelTS - op.startTS) * 1000;
        NSString *url = [AFURLConnectionOperation urlWithOperation:op];
        NSString *opAddr = [NSString stringWithFormat:@"%p", op];
        [[YH_Analytics sharedInstance]logPerformanceWithType:kPerformanceTypeAPICall
                                                  parameters:@{JsonKeyPerformanceEvtEID:opAddr?:@"",
                                                               JsonKeyPerformanceURL: url?:@"",
                                                               JsonKeyPerformanceTS: @((long)(op.startTS*1000)),
                                                               JsonKeyPerformanceCTS:@(deltaTS),
                                                               JsonKeyPerformanceSTATUS : @(0)
                                                               }];
    } error:NULL];
    
}

+ (NSString *)urlWithOperation:(AFURLConnectionOperation *)op
{
    NSString *url = @"";
    if (op && op.request) {
        url = [op.request.URL absoluteString];
        
        BOOL containsAnanlyticURL = ([url rangeOfString:@"yas_mobile"].location == NSNotFound) ? NO : YES;
        
        if ([op.request.HTTPMethod isEqualToString:@"POST"] && !containsAnanlyticURL) {
            NSString *parmeter = [[NSString alloc]initWithData:op.request.HTTPBody encoding:NSUTF8StringEncoding];
            url = [url stringByAppendingString:[NSString stringWithFormat:@"/?%@",parmeter]];
        }
    }
    return url;
}

@end
